@@ -4,6 +4,8 @@ use 5.012;
 use Encode();
 use Time::HiRes();
 
+our $VERSION = '0.1.1';
+
 =head1 NAME
 
 Panda::Lib - Collection of useful functions and classes with Perl and C interface.
@@ -21,8 +23,6 @@ use Panda::Export
 use Panda::Export
     MERGE_COPY => MERGE_COPY_DEST | MERGE_COPY_SOURCE;
     
-our $VERSION = '0.1.0';
-
 require Panda::XSLoader;
 Panda::XSLoader::bootstrap();
 
@@ -114,11 +114,32 @@ Also it contains several C++ classes.
     uint32_t val = string_hash32(str, len);
     uint64_t val = string_hash(str, len);
     
+    
     #include <panda/string.h>
     using panda::string;
     
     string abc("lala");
     ... // everything that std::string supports
+    
+    
+    #include <panda/lib/memory.h>
+    void* mem = StaticMemoryPool<128>::instance()->alloc(); // extremely fast memory allocations
+    void* mem = StaticMemoryPool<128>::threaded_instance()->alloc(); // thread-safe version (still very fast)
+    void* mem = ObjectAllocator::instance()->alloc(256); // dynamic-size fast allocator
+    
+    class MyClass : public AllocatedObject<MyClass> {}
+    MyClass* obj = new MyClass(); // get fast and thread-safe allocations with less code
+
+    class SingleThreadedClass : public AllocatedObject<SingleThreadedClass, false> {} // faster but thread-unsafe
+    
+    // override behaviour from thread-unsafe to thread-safe
+    class MultiThreadedClass : public SingleThreadedClass, public AllocatedObject<MultiThreadedClass> {
+        using AllocatedObject<MultiThreadClass>::operator new;
+        using AllocatedObject<MultiThreadClass>::operator delete;
+    }
+    
+    MemoryPool mypool(32);
+    void* mem = mypool.alloc(); // custom pool with maximum speed, but thread-unsafe
    
 =head1 PERL FUNCTIONS
 
@@ -454,6 +475,138 @@ Detaches string if it's in COW mode. Does nothing otherwise. Returns the string
 
 'ref' has the same meaning as in constructor.
 
+=head2 panda::lib::MemoryPool
+
+Base object for fast memory allocations of particular size (commonly used for small objects allocation).
+This class is thread-unsafe, you can only allocate memory using this object from single thread at one time.
+It is about from 10x to 40x times faster than new+delete.
+
+=head3 METHODS
+
+=head4 MemoryPool (size_t blocksize)
+
+Creates object which allocates blocks of size C<blocksize>. There is no memory overheat, because it doesn't store any additional
+data before/after a memory block. However if you pass C<blocksize> less than 8 bytes, it will still allocate blocks large enough
+to hold 8 bytes.
+
+=head4 void* alloc ()
+
+Allocates new block. Can throw std::bad_alloc if no memory.
+
+=head4 void dealloc (void* ptr)
+
+Returns C<ptr> back to pool. If C<ptr> is a pointer that this object never allocated, the behaviour is undefined.
+
+=head4 ~MemoryPool
+
+Frees internal storage and returns memory to system. All pointers ever allocated by this object become invalid.
+
+=head2 template <int BLOCKSIZE> panda::lib::StaticMemoryPool
+
+This class provides access to singleton memory pool objects for particular block size. It is recommended to use memory pools
+via this interface to reduce memory consumption and fragmentation.
+
+=head3 METHODS
+
+=head4 static MemoryPool* instance ()
+
+Returns MemoryPool object for C<BLOCKSIZE> which is global to the whole process. This object is thread-unsafe.
+
+=head4 static MemoryPool* threaded_instance ()
+
+Returns MemoryPool object for C<BLOCKSIZE> which is global to the current thread. This object is thread-safe.
+Thread safeness is provided by TLS (thread local storage), without any mutexes, rwlocks and so on, so that perfomance is still great.
+
+=head2 panda::lib::ObjectAllocator
+
+Sometimes you don't know the size of a block at compile time and therefore can't use StaticMemoryPool. From the other hand, creating
+MemoryPool objects for particular size every time is expensive. This class provides interface for allocating memory block of an
+arbitrary size. It holds a colletion of MemoryPool objects of various size which are created on-demand.
+
+=head3 METHODS
+
+=head4 ObjectAllocator ()
+
+Creates allocator object. However i would recommend using singleton interface via instance/threaded_instance, see below.
+
+=head4 void* alloc (size_t size)
+
+Allocates block C<size> bytes long. If you pass C<size> less than 8 bytes, it will still allocate 8 bytes.
+
+=head4 void dealloc (void* ptr, size_t size)
+
+Returns C<ptr> back to pool. C<size> is required because MemoryPool doesn't store block sizes before/after blocks to avoid memory
+overheat. If you pass wrong size, or a pointer that was never allocated via this object, the behaviour is undefined.
+
+=head4 ~ObjectAllocator ()
+
+Frees internal storage of all pools and returns memory to system. All pointers ever allocated by this object become invalid.
+
+=head4 static ObjectAllocator* instance ()
+
+Returns ObjectAllocator object which is global to the whole process. This object is thread-unsafe.
+
+=head4 static ObjectAllocator* threaded_instance ()
+
+Returns ObjectAllocator object which is global to the current thread. This object is thread-safe.
+
+=head2 template <class TARGET, bool THREAD_SAFE = true> panda::lib::AllocatedObject
+
+This class is a helper base class. If you inherit from it, objects of your class will be allocated via memory pools instead of
+using default new/delete operators.
+
+Normally, you would need to write this code in order to allocate your objects via MemoryPool:
+
+    class MyClass {
+        static void* operator new (size_t size) {
+            if (size == sizeof(MyClass)) return StaticMemoryPool<sizeof(MyClass)>::threaded_instance()->alloc();
+            return ObjectAllocator::threaded_instance()->alloc(size);
+        }
+        static void operator delete (void* p, size_t size) {
+            if (size == sizeof(MyClass)) StaticMemoryPool<sizeof(MyClass)>::threaded_instance()->dealloc(p);
+            else ObjectAllocator::threaded_instance()->dealloc(p, size);
+        }
+        ...
+    };
+    
+Size check (if/else) is needed to support inheritance, because in that case, size won't match sizeof(MyClass).
+Mostly, programmers use default operator ::new/::delete in case when sizes don't match, however ObjectAllocator can handle
+dynamic sizes and is much faster than default operators, so even in this case we save time.
+
+To avoid writing this code every time, just inherit from AllocatedObject passing your class name as a template parameter.
+You can pass C<false> as a second param to template if you don't need thread-safe allocations to achieve even more perfomance.
+
+    class MyClass : public AllocatedObject<MyClass> { ... }
+
+=head3 TIP
+
+    class MyChild : public MyClass { ... }
+    
+In this case we will still using memory pool, however via dynamic ObjectAllocator which is slightly slower.
+To restore original perfomance redefine new/delete operators again passing your child class name. We will also need to resolve
+multiple inheritance conflicts via C<using> operator.
+
+    class MyChild : public MyClass, public AllocatedObject<MyChild> {
+        using AllocatedObject<MyChild>::operator new;
+        using AllocatedObject<MyChild>::operator delete;
+        ...
+    }
+
+This code will allocate MyChild objects via static memory pool.
+
+=head3 REQUIREMENTS
+
+=over
+
+=item UNIX (any)
+
+For thread-safe instances to work correctly and without memory leaks, you must use C<pthreads> interface to create and join threads.
+As far as i know now everybody use pthreads on UNIX, so this shouldn't be a problem i hope :-)
+
+=item Windows
+
+no requirements
+
 =head1 TYPEMAPS
 
 =head4 panda::string
