@@ -64,13 +64,6 @@ public:
     const char* c_str    () const { return _u.ptr; }
     char*       buf      ()       { retain(); return _u.buf; }
 
-    void dump () {
-        std::printf(
-            "STRDUMP: DYN=%lu, LEN=%lu, CAP=%lu, DATA='%s', SA/BA=%lu/%lu\n",
-            _capacity ? _buf_refcnt() : 0, _length, _capacity, _u.ptr, (unsigned long)this, _capacity ? (unsigned long)_u.buf : 0
-        );
-    }
-
     iterator               begin   ()       { iterator it(buf()); return it; }
     iterator               end     ()       { return iterator(buf() + _length); }
     reverse_iterator       rbegin  ()       { return reverse_iterator(end()); }
